#include <ansi_c.h>
#include <analysis.h>
//==============================================================================
//
// Title:       ACFArrIO
// Purpose:     A short description of the library.
//
// Created on:  2008/4/20 at U 04:39:10 by Hsu-Yang Lee.
// Copyright:   NTU. All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include <utility.h>
#include "ACFArrIO.h"

//==============================================================================
// Constants
#define UNITCONVERT_S2NS 1e9
#define FAILURE 0
#define VERSION_REALACF 20070817

//==============================================================================
// Types

//==============================================================================
// Static global variables

//==============================================================================
// Static functions

//==============================================================================
// Global variables
struct ACF {
	long *delay; /* buffer: delay interphoton times */
	long mdelay; /* buffer size: maximum number of interphoton times to keep in memory*/
	long idelay; /* current position in buffer */
	double dutyFactor; /* maximum duty factor (0-1) per clock period */
	long mtau; /* buffer size: acf function */
//	double *nacf; /* pointer to normalized ACF function array */
	double *dblacf;
//	double *dblacffit;
	double *dblacfx;
	unsigned __int64 acnt, tcnt; /*counts of arr and total timebase pulse, for normalization*/
} d;

//==============================================================================
// Global functions

double ACFFitFunc(double x, double a[], int ncoef)
{
	return(a[0]/(1+x/a[1])/pow((1+x*a[2]/a[1]),0.5)+a[3]);
}

double __stdcall calcRealACFdutyFactor(double m,long c){return( m*c/UNITCONVERT_S2NS);}

long __stdcall calcRealACFmtau(long m, long c){return(m/c);}

void __stdcall setupRealACFArrIO(long mtau, double dutyFactor, const double clock_ns)
{
	long i=0;
	d.mtau=mtau;
	d.dutyFactor=dutyFactor;
	d.idelay=0;
	d.mdelay=(long) (dutyFactor*mtau+1);
	d.delay=(long *)calloc(d.mdelay,sizeof(long));
	d.dblacf=(double *)calloc(mtau+1,sizeof(double));
	d.dblacfx=(double *)calloc(mtau+1,sizeof(double));
	for(i=0;i<mtau+1;i++)d.dblacfx[i]=clock_ns*i;
	d.acnt=0;
	d.tcnt=0;
}


void __stdcall realACFrawArrIO(long *acf, unsigned long tau)
{
	long i=0;
	
	d.tcnt+=tau;
	d.acnt++;
	
	if(d.idelay > d.mdelay)goto error;
	while(d.delay[d.idelay]+tau > d.mtau && d.idelay>0)
	{
		d.delay[d.idelay]=0;
		d.idelay--;
	}
	d.idelay++;
	if(tau>d.mtau) d.idelay=0;
	for(i=d.idelay;i>0;i--) d.delay[i]=d.delay[i-1]+tau;
	for(i=0;i<=d.idelay;i++) acf[d.delay[i]]++;
	return;
	error:
	return;
}
								  
void __stdcall realACFnormArrIO(long *acfarr, double *nacfarr)
{
	long i=0;
	double dblacnt=(double)d.acnt, dbltcnt=(double)d.tcnt;

	for(i=0;i<d.mtau+1;i++) nacfarr[i]=acfarr[i]/pow(dblacnt,2)*pow(dbltcnt,2)/(dbltcnt-i)-1.0;
	return;
}

void __stdcall freeRealACFArrIO(void)
{
	free(d.delay);
	free(d.dblacf);
	free(d.dblacfx);
}

double __stdcall  fitRealACFArrIO(long *acfarr, double *fitarr, double *fitcoefarr, long maxiter,long guess)
{
	double fiterror=0.0;
	double midpt=0.0;
	long uidx=d.mtau, lidx=0, i=0;
	
	if(d.mtau>5)
	{
		for(i=0;i<d.mtau+1;i++)d.dblacf[i]=(double)acfarr[i];
	
		midpt=0.25*d.dblacf[3]+0.75*d.dblacf[d.mtau-3];
		for(i=0;i<guess;i++)
		{
			if(d.dblacf[(long)((uidx+lidx)/2.0)]>midpt)
				lidx=(long)((uidx+lidx)/2.0);
			else
				uidx=(long)((uidx+lidx)/2.0); 
		}
	
		fitcoefarr[1]=d.dblacfx[(long)((uidx+lidx)/2.0)];

		fitcoefarr[3]=d.dblacf[3]; 	
		fitcoefarr[0]=d.dblacf[d.mtau-3]-fitcoefarr[3];
		fitcoefarr[2]=3.0;
		
		NonLinearFitWithMaxIters (d.dblacfx, d.dblacf, fitarr, d.mtau+1, maxiter, ACFFitFunc, fitcoefarr, 4, &fiterror);				   
	}
	return fiterror;
}

void __stdcall getACFcnt(unsigned __int64 *acntpt, unsigned __int64 *tcntpt)
{
	(*acntpt)=d.acnt;
	(*tcntpt)=d.tcnt;
	return;
}

void __stdcall resetrealACF(void){d.idelay=0;}


//==============================================================================
// DLL main entry-point functions

int __stdcall DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    switch (fdwReason) {
        case DLL_PROCESS_ATTACH:
            if (InitCVIRTE (hinstDLL, 0, 0) == 0)
                return 0;     /* out of memory */
            break;
        case DLL_PROCESS_DETACH:
            CloseCVIRTE ();
            break;
    }
    
    return 1;
}

int __stdcall DllEntryPoint (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    /* Included for compatibility with Borland */

    return DllMain (hinstDLL, fdwReason, lpvReserved);
}
