#include <ansi_c.h>
#include <analysis.h>
#include <cvirte.h>
#define UNITCONVERT_S2NS 1e9
#define FAILURE 0
#define VERSION_REALACF 20070817


struct ACF {
	long *delay; /* buffer: delay interphoton times */
	long mdelay; /* buffer size: maximum number of interphoton times to keep in memory*/
	long idelay; /* current position in buffer */
	double dutyFactor; /* maximum duty factor (0-1) per clock period */
	long *acf; /* buffer: auto correlation */
	long mtau; /* buffer size: acf function */
	double *nacf; /* pointer to normalized ACF function array */
	double *dblacf;
	double *dblacffit;
	double *dblacfx;
} d;

struct CCF {
	long *delay; /* buffer: delay interphoton times */
	long mdelay; /* buffer size: maximum number of interphoton times to keep in memory*/
	long idelay; /* current position in buffer */
	double dutyFactor; /* maximum duty factor (0-1) per clock period */
	long *ccf; /* buffer: cross correlation */
	long mtau; /* buffer size: ccf function */
	double *nccf; /* pointer to normalized CCF function array */
	double *dblccf;
	double *dblccffit;
	double *dblccfx;
	long dtau; /* time difference between the two channels */
} g;



int __stdcall DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	switch (fdwReason)
	{
		case DLL_PROCESS_ATTACH:
			if (InitCVIRTE (hinstDLL, 0, 0) == 0)
				return 0;	  /* out of memory */
			break;
		case DLL_PROCESS_DETACH:
			CloseCVIRTE ();
			break;
	}
	
	return 1;
}

int __stdcall DllEntryPoint (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
	/* Included for compatibility with Borland */

	return DllMain (hinstDLL, fdwReason, lpvReserved);
}

double ACFFitFunc(double x, double a[], int ncoef)
{
	return(a[0]/(1+x/a[1])/pow((1+x*a[2]/a[1]),0.5)+a[3]);
}

double __stdcall calcRealACFdutyFactor(double m,long c){return( m*c/UNITCONVERT_S2NS);}

long __stdcall calcRealACFmtau(long m, long c){return(m/c);}

long __stdcall setupRealACF(long mtau,double dutyFactor, double clock_ns){
	long i;
	d.mtau=mtau;
	d.dutyFactor=dutyFactor;
	d.idelay=0;
	d.mdelay=(long) (dutyFactor*mtau+1);
	d.acf=(long *)calloc(mtau+1,sizeof(long));
	d.delay=(long *)calloc( (long) d.mdelay  + 1,sizeof(long));
	d.dblacf=(double *)calloc(mtau+1,sizeof(double));
	d.dblacffit=(double *)calloc(mtau+1,sizeof(double));
	d.dblacfx=(double *)calloc(mtau+1,sizeof(double));
	for (i=0;i<mtau;i++)
		d.dblacfx[i]=i*clock_ns;
	if(d.acf==NULL || d.delay==NULL)goto error;
	d.acf[0]=1;	
	return(d.mdelay);
error:
	printf("setupRealACF: ERROR");
	if(d.acf==NULL)
		printf("memory allocation failed for 'acf' w/mtau=%ld",(long) d.mtau);
	else if(d.delay==NULL)
		printf("memory allocation failed for 'delay' w/mtau=%ld, dutyFactor=%f",
			(long) d.mtau,(double) d.dutyFactor);
	return(FAILURE);
}

long __stdcall realACFraw(long tau){
	int i;
	if(d.idelay > d.mdelay)goto error;
	while(d.delay[d.idelay]+tau > d.mtau && d.idelay>0){
		d.delay[d.idelay]=0;
		d.idelay--;
	}
	d.idelay++;
	if(tau>d.mtau)d.idelay=0;
	for(i=d.idelay;i>0;i--)d.delay[i]=d.delay[i-1]+tau;
	for(i=0;i<=d.idelay;i++)d.acf[d.delay[i]]++;
	return(tau);
error: 
	printf("realACF : ERROR count rate too high for time delay.\n\t"
		"\n\t SUGGESTION: increase dutyFactor by a factor of %.2f or decrease excitation power",
		(double) d.mtau /d.delay[d.idelay-1]);
	return(FAILURE);
}

long __stdcall writeRealACFraw2CSV(char *fnameCSV){
	int i;
	FILE *f=fopen(fnameCSV,"w");
	if(f==NULL)goto error;
	for(i=0;i<d.mtau;i++)fprintf(f,"%ld\n",d.acf[i]);
	printf("writeRealACFtoCSV: acf (%ld pnts) written (1 pnt/line) to %s\n",
		d.mtau,fnameCSV);
	return(d.mtau);
error:
	fprintf(stderr,"writeRealACFtoCSV: ERROR fail to open %s",fnameCSV);
	return(FAILURE);
}

long __stdcall realACFnormalized(void){
	int k;
	double n= (double) d.acf[0];
	d.nacf=(double *)calloc(d.mtau+1,sizeof(double));
	if(d.nacf==NULL)goto error;
	for(k=0;k<d.mtau+1;k++)d.nacf[k]=d.acf[k]/(n-k);
	return(d.acf[0]);
error:
	printf("realACFnormalized: ERROR");
	if(d.nacf==NULL)
		printf("memory allocation failed for 'acfnormalized' w/mtau=%ld",(long) d.mtau);
	return(FAILURE);
}

long __stdcall writeRealACFnormalized2CSV(char *fnameCSV){
	int i;
	FILE *f=fopen(fnameCSV,"w");
	if(f==NULL)goto error;
	for(i=0;i<d.mtau;i++)fprintf(f,"%f\n",d.nacf[i]);
	printf("writeRealACFtoCSV: acf (%ld pnts) written (1 pnt/line) to %s\n",
		d.mtau,fnameCSV);
	return(d.mtau);
error:
	fprintf(stderr,"writeRealACFnormalized2CSV: ERROR fail to open %s",fnameCSV);
	return(FAILURE);
}

long __stdcall ACFnormalized(void){
	long k;
	double n= (double)d.mtau;
	for(k=0;k<d.mtau;k++)d.dblacf[k]=d.acf[k]/(n-k);
	return(d.mtau);
}

double __stdcall  ACFFit1(void)
{
	double fitcoef[4];
	double fiterror;
	double midpt;
	long uidx=d.mtau, lidx=0, i=0;
	
	for(i=0;i<d.mtau;i++)d.dblacf[i]=(double)d.acf[i];
	
	midpt=0.25*d.dblacf[1]+0.75*d.dblacf[d.mtau-1];
	for(i=0;i<20;i++)
	{
		if(d.dblacf[(long)((uidx+lidx)/2.0)]>midpt)
			lidx=(long)((uidx+lidx)/2.0);
		else
			uidx=(long)((uidx+lidx)/2.0); 
	}
	
	fitcoef[1]=d.dblacfx[(long)((uidx+lidx)/2.0)];

	fitcoef[3]=d.dblacf[d.mtau-1]; 	
	fitcoef[0]=d.dblacf[1]-fitcoef[3];
	fitcoef[2]=3.0;
		
	NonLinearFitWithMaxIters (d.dblacfx, d.dblacf, d.dblacffit, d.mtau+1, 1, ACFFitFunc, fitcoef, 4, &fiterror);				   
	return fiterror;
		
}

void _stdcall getrawacf(long *acfraw)
{
	long i;
	for(i=0;i<d.mtau+1;i++)
		acfraw[i]=d.acf[i];
}

void _stdcall getacfnormalized(double *acfnormalized)
{
	long i;
	for(i=0;i<d.mtau+1;i++)
		acfnormalized[i]=d.dblacf[i];
}

void _stdcall getACFFit(double *acfnormalized, double *acfx)
{
	long i;
	for(i=0;i<d.mtau+1;i++)
	{
		acfnormalized[i]=d.dblacffit[i];
		acfx[i]=d.dblacfx[i];
	}
}

void __stdcall realACFrawArrIO(long *delay, long *acf, long tau, long *idelay, long mtau, long mdelay){
	int i;
	if((*idelay) > mdelay)goto error;
	while(delay[(*idelay)]+tau > mtau && (*idelay)>0){
		delay[(*idelay)]=0;
		(*idelay)--;
	}
	(*idelay)++;
	if(tau>mtau)(*idelay)=0;
	for(i=(*idelay);i>0;i--)delay[i]=delay[i-1]+tau;
	for(i=0;i<=(*idelay);i++)acf[delay[i]]++;
	return;
	error:
	return;
}

void __stdcall realACFnormalizedArrIO(double *nacf, long *acf, long mtau){
	int k;
	double n= (double) mtau;
	for(k=0;k<mtau;k++)nacf[k]=acf[k]/(n-k);
	return;
}

void __stdcall ACFsetupArrIO(long arrsize, const double clock_ns)
{
	long i=0;
	d.dblacf=(double *)calloc(arrsize,sizeof(double));
	d.dblacfx=(double *)calloc(arrsize,sizeof(double));
	for(i=0;i<arrsize;i++)d.dblacfx[i]=clock_ns*i;
}

void __stdcall ACFFree(void)
{
	free(d.dblacf);
	free(d.dblacfx);
}

double __stdcall  ACFFit1ArrIO(long *yarr, long arrsize, double *fityarr, double *fitcoef, long iter,long guess)
{
	double fiterror=0.0;
	double midpt=0.0;
	long uidx=arrsize-1, lidx=0, i=0;
	
	if(arrsize>2)
	{
		for(i=0;i<arrsize;i++)d.dblacf[i]=(double)yarr[i];
	
		midpt=0.25*d.dblacf[1]+0.75*d.dblacf[arrsize-2];
		for(i=0;i<guess;i++)
		{
			if(d.dblacf[(long)((uidx+lidx)/2.0)]>midpt)
				lidx=(long)((uidx+lidx)/2.0);
			else
				uidx=(long)((uidx+lidx)/2.0); 
		}
	
		fitcoef[1]=d.dblacfx[(long)((uidx+lidx)/2.0)];

		fitcoef[3]=d.dblacf[arrsize-2]; 	
		fitcoef[0]=d.dblacf[1]-fitcoef[3];
		fitcoef[2]=3.0;
		
		NonLinearFitWithMaxIters (d.dblacfx, d.dblacf, fityarr, arrsize, iter, ACFFitFunc, fitcoef, 4, &fiterror);				   
	}
	return fiterror;
}

void __stdcall file2acf(char *cfn, char *dfn, long mtau, double dutyFactor, double clock_ns)
{
	long i=0, err=0;
	long lngipt=0;
	FILE* fsrc=fopen(cfn, "rb");
	FILE *fdes;
	
	
	setupRealACF(mtau, dutyFactor, clock_ns);
	while(feof(fsrc)==0)
	{
		err=fread (&lngipt, sizeof(long), 1, fsrc);
		realACFraw(lngipt);
	}
	fclose(fsrc);
	fdes=fopen(dfn,"w");
	if(fdes==NULL)goto error;					
	for(i=0;i<d.mtau;i++)fprintf(fdes,"%f\t%d\n",d.dblacfx[i] ,d.acf[i]);
	fclose(fdes);
	return;
error:
	return;
	
}


/*
void __stdcall   ACFFit2(double *yarr, double *xarr, long arrsize, double *fityarr, double *fiterror, long iter)
{
	double fitcoef[4];
	
	fitcoef[3]=yarr[arrsize-1]; 	
	fitcoef[0]=yarr[0]-fitcoef[3];
	fitcoef[1]=xarr[(long)(arrsize/2.0)];
	fitcoef[2]=3.0;
		
	NonLinearFitWithMaxIters (xarr, yarr, fityarr, arrsize, iter, ACFFitFunc, fitcoef, 4, fiterror);				   
		
}
	
void __stdcall   ACFFit3(double *yarr, double *xarr, long arrsize, double *fityarr, double *fiterror, long iter)
{
	double fitcoef[4];
	long i=0, j=0;
	long winwidth=10;
	double *binyarr;
	long binarrsize=0;
	double sum=0.0, maxem=0.0, minem=0.0;
	long maxidx=0, minidx=0;
	
	binarrsize=(long)floor(((double)arrsize)/winwidth);
	binyarr = calloc (binarrsize, sizeof(double));
	for(i=0;i<binarrsize;i++)
	{
		sum=0.0;
		for(j=0;j<winwidth;j++)
			sum+=yarr[i*winwidth+j];
		binyarr[i]=sum;
	}
	Difference (binyarr, binarrsize, 1.0, binyarr[0], binyarr[binarrsize-1], binyarr);
	Abs1D (binyarr, binarrsize, binyarr);
	MaxMin1D (binyarr, binarrsize, &maxem, &maxidx, &minem, &minidx);
	fitcoef[1]=xarr[(long)(maxidx+0.5)*winwidth];
	
	fitcoef[3]=yarr[arrsize-1]; 	
	fitcoef[0]=yarr[0]-fitcoef[3];
	fitcoef[2]=3.0;
		
	NonLinearFitWithMaxIters (xarr, yarr, fityarr, arrsize, iter, ACFFitFunc, fitcoef, 4, fiterror);				   
		
}
*/
		
	
