#include <ansi_c.h>
//==============================================================================
//
// Title:       ipt2tt
// Purpose:     A short description of the library.
//
// Created on:  2008/3/31 at U 03:04:57 by Hsu-Yang Lee.
// Copyright:   NTU. All Rights Reserved.
//
//==============================================================================

//==============================================================================
// Include files

#include <utility.h>
#include "ipt2tt.h"

//==============================================================================
// Constants

//==============================================================================
// Types

//==============================================================================
// Static global variables

//==============================================================================
// Static functions

//==============================================================================
// Global variables

//==============================================================================
// Global functions

struct TT {
	long binWidth; /* width of bin in terms of clock period */
	long time; /* time from start of current bin */
	long count; /* number of photons in current bin */
	long empty; /* number of empty bins that need to be 0 in time trace */
} tt;

long __stdcall calcIpt2tt(long binWidth_ns,long clockPeriod_ns)
{
	long binWidth=binWidth_ns/clockPeriod_ns;
	if(binWidth_ns % clockPeriod_ns != 0)goto error;
	return(binWidth);
error:
	fprintf(stderr,"\ncalcIpt2bin: ERROR requesting binwidth (%ld) [ns]"
		"not integral multiple of clockPeriod %ld [ns]",binWidth_ns,clockPeriod_ns);		
	return(FAILURE);
}

long __stdcall setupIpt2tt(long binWidth)
{
	tt.binWidth=binWidth;
	tt.time=0;
	tt.count=1;
	tt.empty=0;
	return(binWidth);
}

long __stdcall ipt2tt(unsigned long tau, long* emptybins)
{
	/* 3 cases: bin time not up, next bin, many empty bins */
	long count;
	
	tt.time+=tau;
	if(tt.time<tt.binWidth){ /* still within time of single bin */
		tt.count++;
		return(TTBIN_NOT_FULL);
	}else{  /* have left the bin */
		count=tt.count; 
		tt.empty = tt.time / tt.binWidth - 1;
		tt.time%=tt.binWidth;
		tt.count=1;
		(*emptybins)=tt.empty;
		tt.empty=0;
		return(count);
	}
}

long __stdcall getIpt2ttEmptyBins(void)
{
	long empty=tt.empty;
	tt.empty=0;
	return(empty);
}

//==============================================================================
// DLL main entry-point functions

int __stdcall DllMain (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    switch (fdwReason) {
        case DLL_PROCESS_ATTACH:
            if (InitCVIRTE (hinstDLL, 0, 0) == 0)
                return 0;     /* out of memory */
            break;
        case DLL_PROCESS_DETACH:
            CloseCVIRTE ();
            break;
    }
    
    return 1;
}

int __stdcall DllEntryPoint (HINSTANCE hinstDLL, DWORD fdwReason, LPVOID lpvReserved)
{
    /* Included for compatibility with Borland */

    return DllMain (hinstDLL, fdwReason, lpvReserved);
}
