#include <stdio.h>
#include <stdlib.h>
#include "acf.h"
struct ACF {
	unsigned long *delay; /* ptr to interphoton delay times */
	long mdelay; /* maximum # of interphoton times to keep in memory*/
	long idelay; /* current position*/

	unsigned long *acf; /* ptr to acf */
	long mtau; /* acf : maximum delay time in clock cycles */

	double *nacf; /* pointer to normalized ACF function array */	
	unsigned long nPhoton; /* total # of photons [used to normalize ACF]*/
	long double nTime; /* total time (in clock cycles) [used to normalize ACF]*/
} d;

long setupACF(double mdelay_s,double clockPeriod_s,double meanCountRate_cps){
	d.mtau= (long) (mdelay_s/clockPeriod_s);
	d.mdelay=(long) (mdelay_s * meanCountRate_cps);
	d.acf=(unsigned long *)calloc(d.mtau+1,sizeof(long));
	d.delay=(unsigned long *)calloc( (long) d.mdelay+1,sizeof(long));
	if(d.acf==NULL || d.delay==NULL)goto error;
	d.idelay=0;
	d.nPhoton=1;
	d.nTime=0;
	fprintf(stdout,"setupACF: max acf=%ld clock cycles, max photons=%ld\n",
		d.mtau,d.mdelay);
	return(SUCCESS);
error:
	printf("setupACF: ERROR memory allocation failed.");
	return(FAILURE);
}

long updateACF(unsigned long tau){
	long k;
	d.nPhoton++; 
	d.nTime+=tau;
	d.idelay++;
	if(d.idelay > d.mdelay)goto error;
	for(k=d.idelay;k>0;k--)d.delay[k]=d.delay[k-1] + tau;
	while(d.delay[d.idelay]>d.mtau)d.idelay--;
	for(k=d.idelay;k>0;k--)if(d.delay[k]>0)d.acf[d.delay[k]]++;
	return(SUCCESS);
error: 
	fputs("updateACF : ERROR : count rate too high\n\t either decrease laser power"
		" or increase average count rate in software",stderr);
	return(FAILURE);
}

long normalizeACF(void){
	int k;
	double scale;
	if( (d.nacf=(double *)calloc(d.mtau+1,sizeof(double)))==NULL )goto error;
	scale=((double) d.nTime)/ ((double) d.nPhoton);
	scale*=scale;
	if(d.mtau<d.nTime)
		for(k=0;k<d.mtau;k++)d.nacf[k]=d.acf[k]*scale/(d.nTime-k)-1.0;
	else
		for(k=0;k<d.nTime;k++)d.nacf[k]=d.acf[k]*scale/(d.nTime-k)-1.0;
	printf("normalizeACF: totalTime=%Le clock cycles, total Photons=%lu\n",
		d.nTime,d.nPhoton);
	return(SUCCESS);
error:
	printf("normalizeACF: ERROR");
	if(d.nacf==NULL)printf(" memory allocation");
	return(FAILURE);
}	
/* functions to output data, 1 pnt/line in ASCII */
long maxDelayInACF(void){ /* internal */
	int mdelay=d.mtau;
	while(d.acf[mdelay]==0)mdelay--;
	return(mdelay);
}
long writeACFraw2CSV(char *fnameCSV){
	long k, mdelay=maxDelayInACF();
	FILE *f=fopen(fnameCSV,"w");
	if(f==NULL)goto error;
	for(k=0;k<mdelay;k++)fprintf(f,"%lu\n",d.acf[k]);
	printf("writeACFraw2CSV: acf(%ld pnts) written (1 pnt/line) to %s\n",
		mdelay,fnameCSV);
	return(mdelay);
error:
	fprintf(stderr,"writeACFraw2CSV: ERROR fail to open %s",fnameCSV);
	return(FAILURE);
}
long writeACFnorm2CSV(char *fnameCSV){
	long k, mdelay=maxDelayInACF();
	FILE *f=fopen(fnameCSV,"w");
	if(f==NULL)goto error;
	for(k=0;k<mdelay;k++)fprintf(f,"%f\n",d.nacf[k]);
	printf("writeACFnorm2CSV: nacf(%ld pnts) written (1 pnt/line) to %s\n",
		mdelay,fnameCSV);
	return(mdelay);
error:
	fprintf(stderr,"writeACFnorm2CSV: ERROR fail to open %s",fnameCSV);
	return(FAILURE);
}

void cleanACF(void){
	free(d.delay);
	free(d.acf);
	free(d.nacf);
	return;
}
