/* main: to calculate from ipt the ipt histogram, time trace, acf, ccf */
/* Author: (c)J D White */
#define VERSION_CTST  20080814
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "ipt_in.h"
#include "ipt2tt.h"
#include "acf.h"
#include "stats.h"
#include "histo.h"

int main(int argc,char *argv[]){
	IPTdata pnt;
	long width;
	double mdelay_s;
	double clockPeriod_s; 
	double meanCountRate_cps;
	double binWidth_s;
	char fname[32];
	FILE *fIPTin=NULL;
	if(argc!=6)goto error;
	mdelay_s=atof(argv[1]);
	clockPeriod_s=atof(argv[2]); 
	meanCountRate_cps=atof(argv[3]);
	binWidth_s=atof(argv[4]);
	strcpy(fname,argv[5]);
	printf("offline(%d): ipt --> acf, ipt2tt, histogram \n\t"
		"mdelay=%.2e s, clock=%.2e s meanCountRate=%.2ecps\n\t"
		"binWidth=%.2e s (for time trace) Input file:'%s'\n"
		,VERSION_CTST,mdelay_s,clockPeriod_s,meanCountRate_cps,binWidth_s,fname);
	if((fIPTin=openIPTin(fname))==NULL)goto error;
	fclose(fIPTin);
/* 1. Statistics of IPT data*/
	printf("\n1. Calculating Statistics of IPT data...\n");
	if((fIPTin=openIPTin(fname))==NULL)goto error;
	setupStats();
	while((pnt=getIPTin(fIPTin))>0)addpnt2Stats( (long) pnt);
	printf("\tipt[cycles]=[%lu %.2e %lu] std=%.2e total=%.2e[cycles]",
		getStatsMin(),getStatsAvg(),getStatsMax(),getStatsStd(),getStatsSum());
	printf("\n\tipt[s]=[%.2e %.2e %.2e] \n\ttotalTime=%.2e[s] \n\t<CountRate>=%.2e[cps]\n",
		getStatsMin()*clockPeriod_s,getStatsAvg()*clockPeriod_s,
		getStatsMax()*clockPeriod_s,getStatsSum()*clockPeriod_s,
		getStatsBin()/(getStatsSum()*clockPeriod_s) );
	if((getStatsBin()/(getStatsSum()*clockPeriod_s))>meanCountRate_cps){
		meanCountRate_cps=getStatsBin()/(getStatsSum()*clockPeriod_s)*1.2;
		printf("resetting binWidth from user defined to program defined %f",
			meanCountRate_cps);
	}
	
	
	printf("\n2. Calculating IPT histogram (run after stats)...\n");
	if((fIPTin=openIPTin(fname))==NULL)goto error;
	setupHisto(getStatsMax());
	while((pnt=getIPTin(fIPTin))>0)addpnt2Histo(pnt);
	writeHisto("hst.csv");
	printf("\twritten to 'hst.csv'\n");
	clearHisto();
	
	printf("\n3. Calculating ACF...\n"); 
	if((fIPTin=openIPTin(fname))==NULL)goto error;
 	if(setupACF(mdelay_s,clockPeriod_s,meanCountRate_cps)==FAILURE)goto error;
	while((pnt=getIPTin(fIPTin))>0){
		if(updateACF(pnt)==FAILURE)goto error;
	}
	printf("Raw ACF completed.\n");
	normalizeACF();
	writeACFraw2CSV("acf.csv");
	writeACFnorm2CSV("nacf.csv");
	cleanACF();

	printf("\n4. Converting ipt data to time trace (tt)...\n"); 
	if((fIPTin=openIPTin(fname))==NULL)goto error;
	width=setupIpt2tt(binWidth_s,clockPeriod_s,"tt.csv");
	while((pnt=getIPTin(fIPTin))>0)ipt2tt(pnt);
	ipt2tt(width);


	return(1);
error:
	if(argc!=6){
		fprintf(stderr,"offline: offline ipt to fcs, histogram, tt..."
			"\n   INPUT FILE: 'ipt.bin' interphoton arrival time (IPT) data"
			"\n      format: binary native 32 bit long"
			"\n   OUTPUT FILES: '.csv' files containing...."
			"\n            hst.csv  histogram of IPT data"
			"\n            tt.csv time trace"
			"\n            acf.csv raw auto-correlation function"
			"\n            nacf.csv normalized auto-correlation function");
		fprintf(stderr,"\n   COMMAND LINE PARAMETERS:"
			"\n\t maximum delay time for autocorrelation function [s]"
			"\n\t clock Period at which data was recorded [s]"
			"\n\t user estimated mean count rate [counts/second]"
			"\n\t bin Width for time trace [s]"
			"\n\t interphoton input file name");
		return(0);
	}
	fprintf(stderr,"offline Data Display FAILURE: ");
	if(fIPTin==NULL)fprintf(stderr,"\n\tCannot find specified input file for interphoton time data '%s'\n",fname);
	return(0);
}
